package com.blueworld.lassopro;

/**
 *  Wrapper for a primitive type <code>int</code> or <code>long</code>.
 *  Used for returning values from <code>LassoCall</code> methods which in LCAPI 
 *  would require passing parameters of type pointer: <code>int*</code>, 
 *  <code>long*</code> and <code>DescType*</code>.
 *
 * 	<pre>
 *	IntValue outLen = new IntValue();
 *	if ( lasso.arrayGetSize( myArray, outLen ) == ERR_NOERR ) {
 *	    lasso.outputTagData( "Array size: " + outLen.toString() );
 *	}
 * 	</pre>
 */
public class IntValue extends Number
{
	private long value;

	public IntValue()
	{ ; }
	
	public IntValue(int value)
	{
		this.value = (long)value;
	}
	
	public IntValue(long value)
	{
		this.value = value;
	}
	
	/**
	 * Converts the value of this object to a <code>String</code>.
	 */		
	public String toString()
	{
		return Long.toString( this.value );
	}
	
	/**
	 * Sets the value of this object to a specified <code>byte</code>.
	 */
	public void setByte( byte value )
	{
		setLong( (long)value );
	}
	
	/**
	 * Sets the value of this object to a specified <code>short</code>.
	 */	
	public void setShort( short value )
	{
		setLong( (long)value );
	}
	
	/**
	 * Sets the value of this object to a specified <code>int</code>.
	 */		
	public void setInt( int value )
	{
		setLong( (long)value );
	}
	
	/**
	 * Sets the value of this object to a specified <code>long</code>.
	 */	
	public void setLong( long value )
	{
		this.value = value;
	}
		
	/**
	 * Returns the value of this object as <code>boolean</code>.
	 */
	public boolean booleanValue()
	{
		return ( (byte)this.value != 0 );
	}

	/**
	 * Returns the value of this object as a <code>byte</code>.
	 */
	public byte byteValue()  
	{ 
		return (byte)this.value;
	}
		
	/**
	 * Returns the value of this object as a <code>short</code>.
	 */
	public short shortValue()  
	{ 
		return (short)this.value;
	}
	
	/**
	 * Returns the value of this object as an <code>int</code>.
	 */
	public int intValue()  
	{ 
		return (int)this.value;
	}
	
	/**
	 * Returns the value of this object as a <code>long</code>.
	 */
	public long longValue()  
	{ 
		return this.value;
	}

	/**
	 * Sets the value of this object to a specified <code>float</code>.
	 */
	public float floatValue()  
	{ 
		return (float)this.value;
	}
	
	/**
	 * Sets the value of this object to a specified <code>double</code>.
	 */	
	public double doubleValue()  
	{ 
		return (double)this.value;
	}
		
	/**
	 * Casts the value of this object to a four-character <code>String</code>.
	 */		
	public String toDescType()
	{
		return IntToFourCharString ( (int)value );
	}
	
	/**
	 * Casts the value of the specified <code>int</code> to a four-character <code>String</code>.
	 */	
	public static String IntToFourCharString( int v )
	{
		char[] ch = { (char)((v & 0xff000000) >> 24), 
					  (char)((v & 0xff0000) >> 16), 
					  (char)((v & 0xff00) >> 8), 
					  (char)( v & 0xff ) };
		return new String( ch );
	
	}

	
}


