package com.blueworld.lassopro;

/**
 *  Used for creating and manipulating custom Lasso types.
 *  Unlike {@link LassoValue LassoValue} and {@link IntValue IntValue} objects 
 *  which store copies of the data, <code>LassoTypeRef</code> is merely a 
 *  reference to a native object instance. Native objects exist for a fraction 
 *  of a second while Lasso is processing a page, therefore the <code>LassoTypeRef</code> 
 *  objects should never be stored/reused across multiple module invocations.	
 *
 * 	<pre>
 *		LassoTypeRef array = new LassoTypeRef();
 *
 *		if ( lasso.typeAlloc( array, LassoTypeRef.LASSO_ARRAY ) == ERR_NOERR && !array.isNull() ) 
 *		{
 *		    LassoValue val = new LassoValue();
 *		    lasso.typeGetName( array, val );
 *		    lasso.outputTagData( "Type name: " + val.data() );
 *		}
 * 	</pre>
 */
public class LassoTypeRef
{
	public static final String LASSO_NULL 		= "null" 	 ;
	public static final String LASSO_BOOLEAN 	= "boolean"	 ;
	public static final String LASSO_STRING 	= "string" 	 ;
	public static final String LASSO_INTEGER 	= "integer"  ;
	public static final String LASSO_DECIMAL 	= "decimal"  ;
	public static final String LASSO_DATE 		= "date" 	 ;
	public static final String LASSO_PAIR 		= "pair" 	 ;
	public static final String LASSO_ARRAY 		= "array"    ;
	public static final String LASSO_MAP 		= "map" 	 ;
	public static final String LASSO_TAG		= "tag" 	 ;
	
	private int nativeRef;
	{nativeRef = 0;}
	
	/** 
	 * returns <code>true</code> if this object does not refer to a valid type instance.
	 * This could be a result of a failed <code>LassoCall</code> method.
	 */ 
	public boolean isNull()
	{
		return ( nativeRef == 0 );
	}
	
	public String toString()
	{
		return this.getClass().getName() + '@' + Integer.toHexString(this.hashCode()) 
			+ " [0x" + Integer.toHexString(nativeRef) + ']';
	}
}


