package com.blueworld.lassopro;

/**
 *	Used for passing to, and returning values from <code>LassoCall</code> methods.
 *
 * 	<pre>
 *	LassoValue param = new LassoValue();
 *	if ( lasso.getTagParam( 0, param ) == ERR_NOERR ) {
 *	    lasso.outputTagData( "First tag param name: " 
 *                           + param.name() + ", value: " + param.value() );
 *	}
 * 	</pre>
 */
public class LassoValue
{
	/** string */
	public static final int TYPE_CHAR		= 0x54455854; // TEXT
	/** null */
	public static final int TYPE_NULL		= 0x6E756C6C; // null
	/** integer */
	public static final int TYPE_INT		= 0x6C6F6E67; // long
	/** boolean */
	public static final int TYPE_BOOLEAN	= 0x626F6F6C; // bool

	public static final int TYPE_PICT		= 0x50494354; // PICT

	/** binary data */
	public static final int TYPE_BLOB		= 0x626C6F62; // blob
	/** decimal */
	public static final int TYPE_DECIMAL	= 0x646F7562; // doub
	/** date */
	public static final int TYPE_DATETIME	= 0x44617465; // Date
	
	public static final int TYPE_OBJECT		= 0x4F424A5F; // OBJ_
	
	/** array */
	public static final int TYPE_ARRAY		= 0x61727279; // arry
	/** map */
	public static final int TYPE_MAP		= 0x6D61705F; // map_
	/** substitution tag */
	public static final int TYPE_CODE		= 0x636F6465; // code
	/** reference */
	public static final int TYPE_REFERENCE	= 0x7265665F; // ref_
	/** pair */
	public static final int TYPE_PAIR		= 0x70616972; // pair
	/** custom type */
	public static final int TYPE_CUSTOM		= 0x56657273; // Yers
	
	private String name;
	private int nameSize;
	
	private String data;
	private int dataSize;
		
	private int type;

	public LassoValue()
	{
		this.type = TYPE_NULL;
	}

	public LassoValue(int type)
	{
		this.type = type;
	}
		
	public LassoValue(String data)
	{
		this( null, data, data == null ? TYPE_NULL : TYPE_CHAR );
	}

	public LassoValue(String name, String data)
	{
		this( name, data, data == null ? TYPE_NULL : TYPE_CHAR );
	}
	
	/**
	 * Creates new <code>LassoValue</code> instance
	 */
	public LassoValue(String name, String data, int type)
	{
		this.name = name;
		this.data = data;
		this.type = type;
	}

	/**
	 * <code>name</code> field accessor
	 */		
	public String name()
	{
		return this.name;	
	}
	
	/**
	 * <code>data</code> field accessor
	 */
	public String data()
	{
		return this.data;	
	}

	/**
	 * Original type of the data returned from one of the <code>LassoCall</code> methods 
	 * before it was converted to string: <code>TYPE_CHAR</code> for strings, <code>TYPE_INT</code>
	 * for integers, and so on.
	 * 
	 * For unnamed tag parameters, it's a type of the data stored in the <code>data</code> field.
	 * Otherwise, it reflects the type of the <code>name</code> member.
	 */
	public int type()
	{
		return this.type;	
	}
	
	public String setName(String name)
	{
		String temp = this.name;
		this.name = name;
		return temp;
	}
		
	public String setData(String data)
	{
		String temp = this.data;
		this.data = data;
		return temp;
	}

	public int setType(int type)
	{
		int temp = this.type;
		this.type = type;
		return temp;
	}
					
	public String toString()
	{
		return (this.name == null ? "<null>" : this.name) + '=' + (this.data == null ? "<null>" : this.data);	
	}
}


