package com.blueworld.naming;

import javax.naming.*;
import javax.naming.spi.NamingManager;
import java.util.Hashtable;
import java.util.Iterator;

public class ContextImpl
	implements Context
{

	public ContextImpl(LocalRegistry registry, Hashtable env)
	{
		_reg = registry;
		_env = env;
		_name = "";
	}

	public ContextImpl(String name, LocalRegistry registry, Hashtable env)
	{
		_reg = registry;
		_env = env;
		_name = name;
	}

	protected ContextImpl create(String name, LocalRegistry registry, Hashtable env)
	{
		return new ContextImpl(name, registry, env);
	}

	protected final ContextImpl create(LocalRegistry registry, Hashtable env)
	{
		return create("", registry, env);
	}

	public LocalRegistry getRegistry()
	{
		return _reg;
	}

	public Object lookup(String name)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		while(tail != null) 
		{
			String first = parseFirst(tail);
			String rest = parseRest(tail);
			if(first == null)
				return create(getFullPath(name), registry, _env);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
				tail = rest;
			} else
			{
				value = dereference(value, null, registry);
				if(rest == null)
					return value;
				if(value instanceof Context)
					return ((Context)value).lookup(rest);
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}
		return create(getFullPath(name), registry, _env);
	}

	public Object lookup(Name name)
		throws NamingException
	{
		if(name == null)
			return create(_reg, _env);
		LocalRegistry registry = _reg;
		for(int i = 0; i < name.size(); i++)
		{
			String first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(i + 1 == name.size())
					return value;
				if(value instanceof Context)
					return ((Context)value).lookup(name.getSuffix(i + 1));
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		return create(getFullPath(name), registry, _env);
	}

	public Object lookupLink(String name)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		while(tail != null) 
		{
			String first = parseFirst(tail);
			String rest = parseRest(tail);
			if(first == null)
				return create(getFullPath(name), registry, _env);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
				tail = rest;
			} else
			{
				if(rest == null)
					return value;
				value = dereference(value, null, registry);
				if(value instanceof Context)
					return ((Context)value).lookupLink(rest);
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}
		return create(getFullPath(name), registry, _env);
	}

	public Object lookupLink(Name name)
		throws NamingException
	{
		if(name == null)
			return create(_reg, _env);
		LocalRegistry registry = _reg;
		for(int i = 0; i < name.size(); i++)
		{
			String first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				if(i + 1 == name.size())
					return value;
				value = dereference(value, null, registry);
				if(value instanceof Context)
					return ((Context)value).lookupLink(name.getSuffix(i + 1));
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		return create(getFullPath(name), registry, _env);
	}

	public void bind(String name, Object obj)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		String rest;
		Object value;
		do
		{
			String first = parseFirst(tail);
			rest = parseRest(tail);
			if(first == null)
				throw new NamingException("can't bind root");
			if(rest == null)
			{
				value = registry.lookup(first);
				if(value != null)
				{
					throw new NamingException("'" + name + " is already bound to '" + value + "'");
				} else
				{
					registry.bind(first, getReference(registry, obj));
					return;
				}
			}
			value = registry.lookup(first);
			if(!(value instanceof LocalRegistry))
				break;
			registry = (LocalRegistry)value;
			tail = rest;
		} while(true);
		value = dereference(value, null, registry);
		if(value instanceof Context)
		{
			((Context)value).bind(rest, obj);
			return;
		}
		if(value != null)
			throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
		else
			throw new NameNotFoundException(getFullPath(name));
	}

	public void bind(Name name, Object obj)
		throws NamingException
	{
		if(name.size() == 0)
			throw new NamingException("can't bind root");
		LocalRegistry registry = _reg;
		int i;
		String first;
		Object value;
		for(i = 0; i + 1 < name.size(); i++)
		{
			first = name.get(i);
			value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(value instanceof Context)
				{
					((Context)value).bind(name.getSuffix(i + 1), obj);
					return;
				}
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		first = name.get(i);
		value = registry.lookup(first);
		if(value != null)
		{
			throw new NamingException("'" + name + " is already bound to '" + value + "'");
		} else
		{
			registry.bind(first, getReference(registry, obj));
			return;
		}
	}

	public void rebind(String name, Object obj)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		String rest;
		Object value;
		do
		{
			String first = parseFirst(tail);
			rest = parseRest(tail);
			if(first == null)
				throw new NamingException("can't bind root");
			if(rest == null)
			{
				registry.bind(first, getReference(registry, obj));
				return;
			}
			value = registry.lookup(first);
			if(!(value instanceof LocalRegistry))
				break;
			registry = (LocalRegistry)value;
			tail = rest;
		} while(true);
		value = dereference(value, null, registry);
		if(value instanceof Context)
		{
			((Context)value).rebind(rest, obj);
			return;
		}
		if(value != null)
			throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
		else
			throw new NameNotFoundException(getFullPath(name));
	}

	public void rebind(Name name, Object obj)
		throws NamingException
	{
		if(name.size() == 0)
			throw new NamingException("can't bind root");
		LocalRegistry registry = _reg;
		int i;
		String first;
		for(i = 0; i + 1 < name.size(); i++)
		{
			first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(value instanceof Context)
				{
					((Context)value).bind(name.getSuffix(i + 1), obj);
					return;
				}
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		first = name.get(i);
		registry.bind(first, getReference(registry, obj));
	}

	private Object getReference(LocalRegistry registry, Object obj)
	{
		return obj;
	}

	public void unbind(String name)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		String rest;
		Object value;
		do
		{
			String first = parseFirst(tail);
			rest = parseRest(tail);
			if(first == null)
				throw new NamingException("can't unbind root");
			if(rest == null)
				if(registry.lookup(name) instanceof LocalRegistry)
				{
					throw new NamingException("can't unbind subcontext; use destroySubcontext");
				} else
				{
					registry.unbind(first);
					return;
				}
			value = registry.lookup(first);
			if(!(value instanceof LocalRegistry))
				break;
			registry = (LocalRegistry)value;
			tail = rest;
		} while(true);
		value = dereference(value, null, registry);
		if(value instanceof Context)
		{
			((Context)value).unbind(rest);
			return;
		}
		if(value != null)
			throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
		else
			throw new NameNotFoundException(getFullPath(name));
	}

	private Object dereference(Object value, Name tail, LocalRegistry registry)
		throws NamingException
	{
		if(value instanceof ObjectProxy)
			return ((ObjectProxy)value).createObject(_env);
		Context context;
		if(!(value instanceof Reference))
		{
			context = create(registry, _env);
			try 
			{
				return NamingManager.getObjectInstance(value, null, context, _env);
			}
			catch(java.lang.Exception e)
			{
			}
		}
		return value;
	}

	public void unbind(Name name)
		throws NamingException
	{
		if(name.size() == 0)
			throw new NamingException("can't unbind root");
		LocalRegistry registry = _reg;
		int i;
		String first;
		for(i = 0; i + 1 < name.size(); i++)
		{
			first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(value instanceof Context)
				{
					((Context)value).unbind(name.getSuffix(i + 1));
					return;
				}
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		first = name.get(i);
		registry.unbind(first);
	}

	public void rename(String oldName, String newName)
		throws NamingException
	{
		Object obj = lookup(oldName);
		unbind(oldName);
		bind(newName, obj);
	}

	public void rename(Name oldName, Name newName)
		throws NamingException
	{
		Object obj = lookup(oldName);
		unbind(oldName);
		bind(newName, obj);
	}

	public NamingEnumeration list(String name)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		String rest;
		Object value;
		do
		{
			String first = parseFirst(tail);
			rest = parseRest(tail);
			if(first == null)
				return new LNameClassEnumeration(create(registry, _env), registry.list());
			value = registry.lookup(first);
			if(!(value instanceof LocalRegistry))
				break;
			registry = (LocalRegistry)value;
			tail = rest;
		} while(true);
		value = dereference(value, null, registry);
		if(value instanceof Context)
			return ((Context)value).list(rest);
		if(value != null)
			throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
		else
			throw new NameNotFoundException(getFullPath(name));
	}

	public NamingEnumeration list(Name name)
		throws NamingException
	{
		LocalRegistry registry = _reg;
		if(name == null)
			return new LNameClassEnumeration(create(registry, _env), registry.list());
		for(int i = 0; i < name.size(); i++)
		{
			String first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(value instanceof Context)
					return ((Context)value).list(name.getSuffix(i + 1));
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		return new LNameClassEnumeration(create(registry, _env), registry.list());
	}

	public NamingEnumeration listBindings(String name)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		String rest;
		Object value;
		do
		{
			String first = parseFirst(tail);
			rest = parseRest(tail);
			if(first == null)
				return new LBindingEnumeration(create(registry, _env), registry.list());
			value = registry.lookup(first);
			if(!(value instanceof LocalRegistry))
				break;
			registry = (LocalRegistry)value;
			tail = rest;
		} while(true);
		value = dereference(value, null, registry);
		if(value instanceof Context)
			return ((Context)value).list(rest);
		if(value != null)
			throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
		else
			throw new NameNotFoundException(getFullPath(name));
	}

	public NamingEnumeration listBindings(Name name)
		throws NamingException
	{
		LocalRegistry registry = _reg;
		for(int i = 0; name != null && i < name.size(); i++)
		{
			String first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(value instanceof Context)
					return ((Context)value).list(name.getSuffix(i + 1));
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		return new LBindingEnumeration(create(registry, _env), registry.list());
	}

	public Context createSubcontext(String name)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		String rest;
		Object value;
		do
		{
			String first = parseFirst(tail);
			rest = parseRest(tail);
			if(first == null)
				throw new NamingException("can't create root subcontext");
			if(rest == null)
			{
				registry = registry.createSubcontext(first);
				return create(getFullPath(name), registry, _env);
			}
			value = registry.lookup(first);
			if(!(value instanceof LocalRegistry))
				break;
			registry = (LocalRegistry)value;
			tail = rest;
		} while(true);
		value = dereference(value, null, registry);
		if(value instanceof Context)
			return ((Context)value).createSubcontext(rest);
		if(value != null)
			throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
		else
			throw new NameNotFoundException(getFullPath(name));
	}

	public Context createSubcontext(Name name)
		throws NamingException
	{
		if(name.size() == 0)
			throw new NamingException("can't create root subcontext");
		LocalRegistry registry = _reg;
		int i;
		String first;
		for(i = 0; i + 1 < name.size(); i++)
		{
			first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(value instanceof Context)
					return ((Context)value).createSubcontext(name.getSuffix(i + 1));
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		first = name.get(i);
		registry = registry.createSubcontext(first);
		return create(registry, _env);
	}

	public void destroySubcontext(String name)
		throws NamingException
	{
		String tail = name;
		LocalRegistry registry = _reg;
		String rest;
		Object value;
		do
		{
			String first = parseFirst(tail);
			rest = parseRest(tail);
			if(first == null)
				throw new NamingException("can't create root subcontext");
			if(rest == null)
			{
				registry.unbind(first);
				return;
			}
			value = registry.lookup(first);
			if(!(value instanceof LocalRegistry))
				break;
			registry = (LocalRegistry)value;
			tail = rest;
		} while(true);
		value = dereference(value, null, registry);
		if(value instanceof Context)
		{
			((Context)value).destroySubcontext(rest);
			return;
		}
		if(value != null)
			throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
		else
			throw new NameNotFoundException(getFullPath(name));
	}

	public void destroySubcontext(Name name)
		throws NamingException
	{
		if(name.size() == 0)
			throw new NamingException("can't createSubcontext root");
		LocalRegistry registry = _reg;
		int i;
		String first;
		for(i = 0; i + 1 < name.size(); i++)
		{
			first = name.get(i);
			Object value = registry.lookup(first);
			if(value instanceof LocalRegistry)
			{
				registry = (LocalRegistry)value;
			} else
			{
				value = dereference(value, null, registry);
				if(value instanceof Context)
				{
					((Context)value).destroySubcontext(name.getSuffix(i + 1));
					return;
				}
				if(value != null)
					throw new NotContextException(getFullPath(name) + ": expected intermediate context at '" + value + "'");
				else
					throw new NameNotFoundException(getFullPath(name));
			}
		}

		first = name.get(i);
		registry.unbind(first);
	}

	public NameParser getNameParser(String name)
		throws NamingException
	{
		String first = parseFirst(name);
		String rest = parseRest(name);
		if(first == null)
			return new LNameParser(this);
		Object obj = lookupSingleObject(first);
		if(obj instanceof Context)
			return ((Context)obj).getNameParser(rest != null ? rest : "");
		else
			return new LNameParser(this);
	}

	public NameParser getNameParser(Name name)
		throws NamingException
	{
		if(name.size() == 0)
			return new LNameParser(this);
		Object obj = lookupSingleObject(name.get(0));
		if(obj instanceof Context)
			return ((Context)obj).getNameParser(name.getSuffix(1));
		else
			return new LNameParser(this);
	}

	public String composeName(String prefix, String suffix)
		throws NamingException
	{
		return prefix + "/" + suffix;
	}

	public Name composeName(Name prefix, Name suffix)
		throws NamingException
	{
		return prefix.addAll(suffix);
	}

	public String getNameInNamespace()
		throws NamingException
	{
		throw new OperationNotSupportedException();
	}

	private Object lookupSingleObject(String name)
		throws NamingException
	{
		Object obj = lookupSingle(name);
		if(obj instanceof ObjectProxy)
			return ((ObjectProxy)obj).createObject(_env);
		else
			return obj;
	}

	protected Object lookupSingle(String name)
		throws NamingException
	{
		throw new UnsupportedOperationException();
	}

	protected void rebindSingle(String name, Object obj)
		throws NamingException
	{
		throw new UnsupportedOperationException();
	}

	protected void unbindSingle(String name)
		throws NamingException
	{
		throw new UnsupportedOperationException();
	}

	protected Context createSingleSubcontext(String name)
		throws NamingException
	{
		throw new UnsupportedOperationException();
	}

	protected void destroySingleSubcontext(String name)
		throws NamingException
	{
		unbindSingle(name);
	}

	protected Iterator listSingle()
	{
		throw new UnsupportedOperationException();
	}

	protected String parseFirst(String name)
		throws NamingException
	{
		if(name == null || name.equals(""))
			return null;
		int p = name.indexOf(getSeparator());
		if(p == 0)
			return parseFirst(name.substring(1));
		if(p > 0)
			return name.substring(0, p);
		else
			return name;
	}

	protected String parseRest(String name)
		throws NamingException
	{
		if(name == null || name.equals(""))
			return null;
		int p = name.indexOf(getSeparator());
		if(p == 0)
			return parseRest(name.substring(1));
		if(p > 0)
			return name.substring(p + 1);
		else
			return null;
	}

	protected char getSeparator()
	{
		return '/';
	}

	protected String getSeparatorString()
	{
		return "/";
	}

	protected String getFullPath(String name)
	{
		if(_name == null || _name.equals(""))
			return name;
		if(name == null)
			return _name;
		String sep;
		for(sep = getSeparatorString(); name.endsWith(sep); name = name.substring(0, name.length() - sep.length()));
		if(name.equals(""))
			return _name;
		if(name.startsWith(sep))
			return _name + name;
		else
			return _name + sep + name;
	}

	protected String getFullPath(Name name)
	{
		if(_name == null || _name.equals(""))
			return name.toString();
		if(name == null || name.size() == 0)
		{
			return _name;
		} else
		{
			String sep = getSeparatorString();
			return _name + sep + name;
		}
	}

	public Object addToEnvironment(String prop, Object value)
		throws NamingException
	{
		Object old = _env.get(prop);
		_env.put(prop, value);
		return old;
	}

	public Object removeFromEnvironment(String prop)
		throws NamingException
	{
		Object old = _env.get(prop);
		_env.remove(prop);
		return old;
	}

	public Hashtable getEnvironment()
		throws NamingException
	{
		return _env;
	}

	public void close()
		throws NamingException
	{
	}

	public String toString()
	{
		return "[ContextImpl " + _name + "]";
	}

	protected static Class clazz;
	protected Hashtable _env;
	protected LocalRegistry _reg;
	protected String _name;

	static 
	{
		clazz = com.blueworld.naming.ContextImpl.class;
	}
}
