package com.blueworld.naming;

import java.io.PrintStream;
import java.util.*;
import javax.naming.*;

public class LName
	implements Name
{

	public LName(Context context)
	{
		items = new ArrayList();
		this.context = context;
	}

	public LName(Context context, String first)
	{
		items = new ArrayList();
		this.context = context;
		if(first != null)
			items.add(first);
	}

	public LName(Context context, String first, String rest)
	{
		items = new ArrayList();
		this.context = context;
		if(first != null)
			items.add(first);
		if(rest != null)
			items.add(rest);
	}

	public Object clone()
	{
		LName name = new LName(context);
		for(int i = 0; i < items.size(); i++)
			name.items.add(items.get(i));

		return name;
	}

	public int size()
	{
		return items.size();
	}

	public boolean isEmpty()
	{
		return items.size() == 0;
	}

	public Enumeration getAll()
	{
		return Collections.enumeration(items);
	}

	public String get(int pos)
	{
		if(pos < items.size())
			return (String)items.get(pos);
		else
			return null;
	}

	public Name getPrefix(int posn)
	{
		LName name = new LName(context);
		for(int i = 0; i < posn; i++)
			name.items.add(items.get(i));

		return name;
	}

	public Name getSuffix(int posn)
	{
		Context subcontext = context;
		int i = 0;
		do
		{
			if(i >= posn)
				break;
			String item = (String)items.get(i);
			try
			{
				Object obj = subcontext.lookup(item);
				if(!(obj instanceof Context))
					break;
				subcontext = (Context)obj;
			}
			catch(NamingException e)
			{
				break;
			}
			i++;
		} while(true);
		LName name = new LName(subcontext);
		for(i = posn; i < items.size(); i++)
		{
			String item = (String)items.get(i);
			name.items.add(items.get(i));
		}

		return name;
	}

	public boolean startsWith(Name name)
	{
		if(name == null)
			return false;
		if(size() < name.size())
			return false;
		for(int i = 0; i < name.size(); i++)
		{
			System.out.println(i + ": " + get(i) + " " + name.get(i));
			if(!get(i).equals(name.get(i)))
				return false;
		}

		return true;
	}

	public boolean endsWith(Name name)
	{
		if(name == null)
			return false;
		int nameSize = name.size();
		if(size() < nameSize)
			return false;
		int offset = size() - nameSize;
		for(int i = 0; i < nameSize; i++)
			if(!get(i + offset).equals(name.get(i)))
				return false;

		return true;
	}

	public Name addAll(Name suffix)
		throws InvalidNameException
	{
		for(int i = 0; i < suffix.size(); i++)
			items.add(suffix.get(i));

		return this;
	}

	public Name addAll(int posn, Name suffix)
		throws InvalidNameException
	{
		for(int i = 0; i < suffix.size(); i++)
			items.add(posn, suffix.get(i));

		return this;
	}

	public Name add(String comp)
		throws InvalidNameException
	{
		items.add(comp);
		return this;
	}

	public Name add(int posn, String comp)
		throws InvalidNameException
	{
		items.add(posn, comp);
		return this;
	}

	public Object remove(int posn)
		throws InvalidNameException
	{
		items.remove(posn);
		return this;
	}

	public int hashCode()
	{
		int hashCode = 337;
		for(int i = size() - 1; i >= 0; i--)
			hashCode = 65521 * hashCode + get(i).hashCode();

		return hashCode;
	}

	public boolean equals(Object obj)
	{
		if(!(obj instanceof Name))
			return false;
		Name name = (Name)obj;
		if(size() != name.size())
			return false;
		for(int i = size() - 1; i >= 0; i--)
			if(!get(i).equals(name.get(i)))
				return false;

		return true;
	}

	public int compareTo(Object rawB)
	{
		if(!(rawB instanceof Name))
			return -1;
		Name b = (Name)rawB;
		for(int i = 0; i < size(); i++)
		{
			if(i >= b.size())
				return 1;
			String sa = get(i);
			String sb = b.get(i);
			int cmp = sa.compareTo(sb);
			if(cmp != 0)
				return cmp;
		}

		return size() != b.size() ? -1 : 0;
	}

	public String toString()
	{
		String name = null;
		for(int i = 0; i < size(); i++)
		{
			String str = get(i);
			if(name != null)
				try
				{
					name = context.composeName(name, str);
				}
				catch(NamingException e)
				{
					name = name + "/" + str;
				}
			else
				name = str;
		}

		return name != null ? name : "";
	}
	
	protected Context context;
	private ArrayList items;

}
