package com.omnipilot.lassopro;

/**
* Constants for the various error codes which can be returned by your module.
*/
public final class LassoErrors
{
	private LassoErrors() {;}
	
	/** success result code  */
	public static final int NO_ERR 					= 0;
	/** failed assertion */
	public static final int Assert 					= -10000;

	// stream errors	
	/** could not write to stream */
	public static final int StreamReadError			= -9999;
	/** could not read from stream */
	public static final int StreamWriteError		= -9998;

	// memory errors
	/** generic memory error */
	public static final int Memory					= -9997;
	/** invalid memory object */
	public static final int InvalidMemoryObject		= -9996;
	/** not enough memory */
	public static final int OutOfMemory				= -9995;
	/** stack overflow error */
	public static final int OutOfStackSpace			= -9994;
	/** error disposing an object */
	public static final int CouldNotDisposeMemory	= -9991;

	// file errors
	
	/** generic file error */
	public static final int File					= -9990;
	/** trying to work with an invalid file */
	public static final int FileInvalid				= -9989;
	/** trying to access a file in a mode that it doesn't support */
	public static final int FileInvalidAccessMode	= -9988;
	/**  could not create or open the file */
	public static final int CouldNotCreateOrOpenFile= -9987;
	/** could not close the file */
	public static final int CouldNotCloseFile		= -9986;
	/** could not delete the file */
	public static final int CouldNotDeleteFile		= -9985;
	/** file does not exist */
	public static final int FileNotFound			= -9984;
	/** trying to create a file that already exists */
	public static final int FileAlreadyExists		= -9983;
	/** file is corrupted */
	public static final int FileCorrupt				= -9982;
	/** bad volume name */
	public static final int VolumeDoesNotExist		= -9981;
	/** no room left on disk */
	public static final int DiskFull				= -9980;
	/** no more items allowed inthe directory */
	public static final int DirectoryFull			= -9979;
	/** I/O error */
	public static final int IOError					= -9978;
	/** pathname is invalid */
	public static final int InvalidPathname			= -9977;
	/** filename is invalid */
	public static final int InvalidFilename			= -9976;
	/** file is locked */
	public static final int FileLocked				= -9975;
	/** file is unlocked */
	public static final int FileUnlocked			= -9974;
	/** file is open */
	public static final int FileIsOpen				= -9973;
	/** file is closed */
	public static final int FileIsClosed			= -9972;
	/** beginning of file reached */
	public static final int BOF						= -9971;
	/** end of file reached */
	public static final int EOF						= -9970;
	/** unable to complete a write operation to the file */
	public static final int CouldNotWriteToFile		= -9969;
	/** unable to complete a read operation from the file */
	public static final int CouldNotReadFromFile	= -9968;

	// resource errors
	/** resource not found */
	public static final int ResNotFound				= -9967;
	/** generic resource error */
	public static final int Resource				= -9966;

	// networking errors
	
	/** unknown networking error */
	public static final int Network					= -9965;
	
	// security violation errors
	
	/** the username supplied for the action is not valid */
	public static final int InvalidUsername			= -9964;
	/** the password supplied for the action is not valid */
	public static final int InvalidPassword			= -9963;
	/** the database name supplied is not valid */
	public static final int InvalidDatabase			= -9962;
	/** general permissions error */
	public static final int NoPermission			= -9961;
	/** the specified action is restricted */
	public static final int FieldRestriction		= -9960;
	
	// contiguous web errors
	
	/** add record error */ 
	public static final int WebAddError				= -9959;
	/** update record error */ 
	public static final int WebUpdateError			= -9958;
	/** delete record error */ 
	public static final int WebDeleteError			= -9957;
		
	// general errors
	
	/** an invalid parameter was passed to a function */
	public static final int InvalidParameter		= -9956;
	/** some memory passed to a function was too small to hold the results */
	public static final int Overflow				= -9955;
	/** a pointer was nil when it shouldn't have been */
	public static final int NilPointer				= -9954;
	/** default when none of the cross-platform errors seem to fit */
	public static final int UnknownError			= -9953;
	
	// formatting errors
	
	/** a looping tag was aborted; all looping tags must catch this exception */
	public static final int FormattingLoopAborted	= -9952;
	/** bad syntax used in a format file; parsing of the file was aborted */
	public static final int FormattingSyntaxError	= -9951;

	// non contiguous web errors (backwards compatible)
	
	/** value missing for required field for Add */
	public static final int WebRequiredFieldMissing = -800;
	/** adding repeating related fields isn't supported */
	public static final int WebRepeatingRelatedField = -801;
	/** no records found */
	public static final int WebNoSuchObject 		= -1728;
	/** operation timed out */
	public static final int WebTimeout 				= -1712;
	/** action not supported */
	public static final int WebActionNotSupported	= -802;
	/** the specified database was not found */
	public static final int WebConnectionInvalid 	= -609;
	/** the module was not found */
	public static final int WebModuleNotFound 		= -2000;
	/** the file was not found */
	public static final int HTTPFileNotFound 		= 404;
	/** third-party generic datasource error */
	public static final int DatasourceError 		= -3000;
	
}


