package com.omnipilot.lassopro;


/**
 * Base class for any tag module.
 * Most tag modules output data onto the web page, though some tags may
 * perform other actions based on the parameters passed to them.<br><br>
 *
 * Every <code>LassoTagModule</code> must implement {@link LassoJavaModule#registerLassoModule registerLassoModule} 
 * method, and one or more methods with the same signature as {@link #TAG_METHOD_PROTOTYPE TAG_METHOD_PROTOTYPE}.<br><br>
 *
 * Lasso calls <code>registerLassoModule</code> once at startup to give this
 * module a chance to register its tags. <code>LassoTagModule</code> must then 
 * call <code>registerTagModule</code> as many times as there are tags implemented by this module.
 */
public abstract class LassoTagModule 
	extends com.blueworld.lassopro.LassoTagModule
{
	protected LassoTagModule()
	{
		super();
	}
	
/**
  * Use this method to register substitution tags implemented by your module.
  * You should call <code>registerTagModule</code> as many times as there are tags implemented in your module.
  *
  * @param moduleName name of the module as returned by <code>[Lasso_TagModuleName]</code> LDML tag
  * @param nameSpace namespace this tag will be registered under
  * @param tagName name of the custom LDML tag implemented by this module
  * @param methodName Lasso will call this method every time it finds <code>tagName</code> tag in a <code>xxx.lasso</code> format file
  * @param flags one or more logically or'ed <code>FLAG</code> constants
  * @param description optional tag info, such as brief description of this tag usage
  */
  
	protected native void registerTagModule(String moduleName, String nameSpace, String tagName, String methodName, int flags, String description);
	
	protected void registerTagModule(String moduleName, String tagName, String methodName, int flags, String description)
	{
		registerTagModule(moduleName, "", tagName, methodName, flags, description);
	}
	
	
	/*
	protected void registerTagModule(String moduleName, String nameSpace,  String tagName, String methodName, int flags, String description)
	{
		registerTagModule(moduleName, tagName, methodName, flags, description);
	}
	*/
}
