package com.omnipilot.lassopro;

/**
	Used for creating and manipulating custom Lasso types.
	
	This class is used by the developer as an opaque handle to an internally allocated Variant. 
	Variants allocated in this manner are garbage collected by the API at the end of the current LJAPI call. 
	This means that <code>LassoTypeRef</code>'s are not usable in across subsequent LJAPI invocations. 
	The method of garbage collection will be the same as that currently used for <code>LassoValue</code>'s 
	in that the current {@link LassoCall LassoCall} context retains a list of all Variants allocated,
	and destroys each of them after the return of the current LJAPI call.

	<pre>
		LassoTypeRef array = new LassoTypeRef();
	
		if ( lasso.typeAlloc( array, LassoTypeRef.LASSO_ARRAY ) == ERR_NOERR && !array.isNull() ) 
		{
		    LassoValue val = new LassoValue();
		    lasso.typeGetName( array, val );
		    lasso.outputTagData( "Type name: " + val.data() );
		}
	</pre>
 */
public class LassoTypeRef
	extends com.blueworld.lassopro.LassoTypeRef
{

}


