package com.omnipilot.util;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import com.omnipilot.util.LassoException;

public class Introspect
{
	// *****************************************************************
	// locate property-setter method
	// *****************************************************************
	public static Method getSetMethod(java.beans.BeanInfo info, String propertyName)
	{
		java.beans.PropertyDescriptor pds[] = info.getPropertyDescriptors();
		Method method = null;
		for(int i = 0; i < pds.length; i++)
		{
			if(!pds[i].getName().equals(propertyName) || pds[i].getWriteMethod() == null)
				continue;
			method = pds[i].getWriteMethod();
			if(method.getParameterTypes()[0].equals(java.lang.String.class))
				return method;
		}

		if(method != null)
			return method;
		else
			return getSetMethod(info.getBeanDescriptor().getBeanClass(), propertyName);
	}

	public static Method getSetMethod(Class cl, String propertyName)
	{
		Method method = getSetMethod(cl, propertyName, false);
		if(method != null)
			return method;
		else
			return getSetMethod(cl, propertyName, true);
	}

	public static Method getSetMethod(Class cl, String propertyName, boolean ignoreCase)
	{
		String setName = "set" + propertyNameToMethodName(propertyName);
		Method bestMethod = null;
		for(Class ptrCl = cl; ptrCl != null; ptrCl = ptrCl.getSuperclass())
		{
			Method method = getSetMethod(ptrCl.getMethods(), setName, ignoreCase);
			if(method != null && method.getParameterTypes()[0].equals(java.lang.String.class))
				return method;
			if(method != null)
				bestMethod = method;
		}

		if(bestMethod != null)
			return bestMethod;
		Class interfaces[] = cl.getInterfaces();
		for(int i = 0; i < interfaces.length; i++)
		{
			Method method = getSetMethod(interfaces[i].getMethods(), setName, ignoreCase);
			if(method != null && method.getParameterTypes()[0].equals(java.lang.String.class))
				return method;
			if(method != null)
				bestMethod = method;
		}

		if(bestMethod != null)
			return bestMethod;
		else
			return null;
	}
	
	private static Method getSetMethod(Method methods[], String setName, boolean ignoreCase)
	{
		for(int i = 0; i < methods.length; i++)
		{
			Method method = methods[i];
			if((ignoreCase || method.getName().equals(setName)) && (!ignoreCase || method.getName().equalsIgnoreCase(setName)) && Modifier.isPublic(method.getModifiers()) && Modifier.isPublic(method.getDeclaringClass().getModifiers()) && method.getParameterTypes().length == 1 && method.getReturnType().equals(Void.TYPE))
				return method;
		}

		return null;
	}

	// *****************************************************************
	// locate property-getter method
	// *****************************************************************
	
	public static Method getGetMethod(java.beans.BeanInfo info, String propertyName)
	{
		java.beans.PropertyDescriptor pds[] = info.getPropertyDescriptors();
		for(int i = 0; i < pds.length; i++)
			if(pds[i].getName().equals(propertyName) && pds[i].getReadMethod() != null && Modifier.isPublic(pds[i].getReadMethod().getDeclaringClass().getModifiers()))
				return pds[i].getReadMethod();

		return getGetMethod(info.getBeanDescriptor().getBeanClass(), propertyName);
	}

	public static Method getGetMethod(Class cl, String propertyName)
	{
		Method method = getGetMethod(cl, propertyName, false);
		if(method != null)
			return method;
		else
			return getGetMethod(cl, propertyName, true);
	}

	public static Method getGetMethod(Class cl, String propertyName, boolean ignoreCase)
	{
		String getName = "get" + propertyNameToMethodName(propertyName);
		String isName = "is" + propertyNameToMethodName(propertyName);
		for(Class ptrCl = cl; ptrCl != null; ptrCl = ptrCl.getSuperclass())
		{
			Method method = getGetMethod(ptrCl.getMethods(), getName, isName, ignoreCase);
			if(method != null)
				return method;
		}

		Class interfaces[] = cl.getInterfaces();
		for(int i = 0; i < interfaces.length; i++)
		{
			Method method = getGetMethod(interfaces[i].getMethods(), getName, isName, ignoreCase);
			if(method != null)
				return method;
		}

		return null;
	}

	private static Method getGetMethod(Method methods[], String getName, String isName, boolean ignoreCase)
	{
		for(int i = 0; i < methods.length; i++)
		{
			Method method = methods[i];
			if(!Modifier.isPublic(method.getModifiers()) || !Modifier.isPublic(method.getDeclaringClass().getModifiers()) || method.getParameterTypes().length != 0 || method.getReturnType().equals(Void.TYPE))
				continue;
			if(!ignoreCase && methods[i].getName().equals(getName))
				return methods[i];
			if(ignoreCase && methods[i].getName().equalsIgnoreCase(getName))
				return methods[i];
			if(!methods[i].getReturnType().equals(Boolean.TYPE))
				continue;
			if(!ignoreCase && methods[i].getName().equals(isName))
				return methods[i];
			if(ignoreCase && methods[i].getName().equalsIgnoreCase(isName))
				return methods[i];
		}

		return null;
	}
	
	public static String propertyNameToMethodName(String propertyName)
	{
		char ch = propertyName.charAt(0);
		if(Character.isLowerCase(ch))
			propertyName = Character.toUpperCase(ch) + propertyName.substring(1);
		return propertyName;
	}

	public static String methodNameToPropertyName(java.beans.BeanInfo info, String methodName)
	{
		java.beans.PropertyDescriptor pds[] = info.getPropertyDescriptors();
		for(int i = 0; i < pds.length; i++)
		{
			if(pds[i].getReadMethod() != null && pds[i].getReadMethod().getName().equals(methodName))
				return pds[i].getName();
			if(pds[i].getWriteMethod() != null && pds[i].getWriteMethod().getName().equals(methodName))
				return pds[i].getName();
		}

		return methodNameToPropertyName(methodName);
	}

	public static String methodNameToPropertyName(String methodName)
	{
		if(methodName.startsWith("get"))
			methodName = methodName.substring(3);
		else
		if(methodName.startsWith("set"))
			methodName = methodName.substring(3);
		else
		if(methodName.startsWith("is"))
			methodName = methodName.substring(2);
		if(methodName.length() == 0)
			return null;
		char ch = methodName.charAt(0);
		if(Character.isUpperCase(ch) && (methodName.length() == 1 || !Character.isUpperCase(methodName.charAt(1))))
			methodName = Character.toLowerCase(ch) + methodName.substring(1);
		return methodName;
	}
	
	
	// *****************************************************************
	// functions to convert built-in Java data types to string.
	// *****************************************************************

	public static String toString(Object o) 
	{
		return (o == null) ? "" : o.toString();
	}

	public static String toString(byte b) 
	{
		return new Byte(b).toString();
	}

	public static String toString(boolean b) 
	{
		return new Boolean(b).toString();
	}

	public static String toString(short s) 
	{
		return new Short(s).toString();
	}

	public static String toString(int i) 
	{
		return new Integer(i).toString();
	}

	public static String toString(float f) 
	{
		return new Float(f).toString();
	}

	public static String toString(long l) 
	{
		return new Long(l).toString();
	}

	public static String toString(double d) 
	{
		return new Double(d).toString();
	}

	public static String toString(char c) 
	{
		return new Character(c).toString();
	}	
	
	// *****************************************************************
	// functions to convert string to various built-in Java data types.
	// *****************************************************************
	
	public static boolean coerceToBoolean(String s) 
	{
		if (s == null || s.length() == 0)
			return false;
		else
			return Boolean.valueOf(s).booleanValue();
	}

	public static byte coerceToByte(String s) 
	{
		if (s == null || s.length() == 0)
			return (byte) 0;
		else
			return Byte.valueOf(s).byteValue();
	}

	public static char coerceToChar(String s) 
	{
		if (s == null || s.length() == 0) 
			return (char) 0;
		else 
			// this trick avoids escaping issues
			return (char)(int) s.charAt(0);
	}

	public static double coerceToDouble(String s) 
	{
		if (s == null || s.length() == 0)
			return (double) 0;
		else
			return Double.valueOf(s).doubleValue();
	}

	public static float coerceToFloat(String s) 
	{
		if (s == null || s.length() == 0)
			return (float) 0;
		else
			return Float.valueOf(s).floatValue();
	}

	public static int coerceToInt(String s) 
	{
		if (s == null || s.length() == 0)
			return 0;
		else
			return Integer.valueOf(s).intValue();
	}

	public static short coerceToShort(String s) 
	{
		if (s == null || s.length() == 0)
			return (short) 0;
		else
			return Short.valueOf(s).shortValue();
	}

	public static long coerceToLong(String s) 
	{
		if (s == null || s.length() == 0)
			return (long) 0;
		else
			return Long.valueOf(s).longValue();
	}

	// *****************************************************************
	// convert string to a Java object.
	// *****************************************************************
	public static Object coerce(String s, Class target) 
	{
		boolean isNullOrEmpty = (s == null || s.length() == 0);
	
		if (target == Boolean.class) 
		{
			if (isNullOrEmpty) 
				s = "false";
			return new Boolean(s);
		} 
		else if (target == Byte.class) 
		{
			if (isNullOrEmpty)
				return new Byte((byte) 0);
			else
				return new Byte(s);
		} 
		else if (target == Character.class) 
		{
			if (isNullOrEmpty)
				return new Character((char) 0);
			else 
				return new Character(s.charAt(0));
		} 
		else if (target == Double.class) 
		{
			if (isNullOrEmpty)
				return new Double(0);
			else
				return new Double(s);
		} 
		else if (target == Float.class) 
		{
			if (isNullOrEmpty)
				return new Float(0);
			else
				return new Float(s);
		} 
		else if (target == Integer.class) 
		{
			if (isNullOrEmpty)
				return new Integer(0);
			else
				return new Integer(s);
		} 
		else if (target == Short.class) 
		{
			if (isNullOrEmpty)
				return new Short((short) 0);
			else
				return new Short(s);
		} 
		else if (target == Long.class) 
		{
			if (isNullOrEmpty)
				return new Long(0);
			else
				return new Long(s);
		} 
		else 
		{
			return null;
		}
	}

	// *****************************************************************
	// convert array of params to method args
	// *****************************************************************
	public static Object[] coerce(Class parms[], Object args[]) 
	{
		Object result[] = args;
		for (int i=0; i<args.length; i++) 
		{
			if (args[i] instanceof byte[] && !parms[i].isArray()) 
			{
				Class c = parms[i];
				String s = new String((byte[])args[i]);
				result[i] = s;
				try 
				{
					if (c == Boolean.TYPE) result[i]=new Boolean(s);
					if (c == Byte.TYPE) result[i]=new Byte(s);
					if (c == Short.TYPE) result[i]=new Short(s);
					if (c == Integer.TYPE) result[i]=new Integer(s);
					if (c == Float.TYPE) result[i]=new Float(s);
					if (c == Long.TYPE) result[i]=new Long(s);
					if (c == Character.TYPE && s.length()>0) 
						result[i]=new Character(s.charAt(0));
				}
				catch (NumberFormatException n) 
				{
					//oh well, we tried!
				}
			}
			else if (args[i] instanceof Number && parms[i].isPrimitive()) 
			{
				if (result==args) result=(Object[])result.clone();
				Class c = parms[i];
				Number n = (Number)args[i];
				if (c == Boolean.TYPE) result[i]=new Boolean(0.0!=n.floatValue());
				if (c == Byte.TYPE) result[i]=new Byte(n.byteValue());
				if (c == Short.TYPE) result[i]=new Short(n.shortValue());
				if (c == Integer.TYPE) result[i]=new Integer(n.intValue());
				if (c == Float.TYPE) result[i]=new Float(n.floatValue());
				if (c == Long.TYPE && !(n instanceof Long)) 
					result[i]=new Long(n.longValue());
			}
			else if (args[i] instanceof java.util.Hashtable && parms[i].isArray()) 
			{
				try 
				{
					java.util.Hashtable ht = (java.util.Hashtable)args[i];
					int size = ht.size();
	
					//Verify that the keys are Long, and determine maximum
					for (java.util.Enumeration e = ht.keys(); e.hasMoreElements(); ) 
					{
						int index = ((Long)e.nextElement()).intValue();
						if (index >= size) size = index+1;
					}
	
					Object tempArray[] = new Object[size];
					Class tempTarget[] = new Class[size];
					Class targetType = parms[i].getComponentType();
	
					//flatten the hash table into an array
					for (int j=0; j<size; j++) 
					{
						tempArray[j] = ht.get(new Long(j));
						if (tempArray[j] == null && targetType.isPrimitive()) 
							throw new Exception("bail");
						tempTarget[j] = targetType;
					}
	
					//coerce individual elements into the target type
						Object coercedArray[] = coerce(tempTarget, tempArray);
	
					//copy the results into the desired array type
						Object array = java.lang.reflect.Array.newInstance(targetType,size);
					for (int j=0; j<size; j++) 
					{
						java.lang.reflect.Array.set(array, j, coercedArray[j]);
					}
	
					result[i]=array;
				}
				catch (Exception e) 
				{
					//leave result[i] alone...
				}
			}
		}
		return result;
	}
	
	//*********************************************************************
	// PropertyEditor Support
	//*********************************************************************
        		
	public static Object getValueFromBeanInfoPropertyEditor ( 
		Class attrClass, String attrName, String attrValue, Class propertyEditorClass ) throws LassoException 
	{
		try 
		{
			java.beans.PropertyEditor pe = (java.beans.PropertyEditor)propertyEditorClass.newInstance();
			pe.setAsText(attrValue);
			return pe.getValue();
		} 
		catch (Exception ex) 
		{
			throw new LassoException( "Bean property conversion error: " + ex.getMessage() 
				+ "(value=" + attrValue + ", class=" + attrClass.getName() + ", attribute=" + attrName + ")");
		}
	}

	public static Object getValueFromPropertyEditorManager ( 
		Class attrClass, String attrName, String attrValue) throws LassoException 
	{
		try 
		{
			java.beans.PropertyEditor propEditor = java.beans.PropertyEditorManager.findEditor(attrClass);
			if (propEditor != null) 
			{
				propEditor.setAsText(attrValue);
				return propEditor.getValue();
			} 
			else 
			{
				throw new IllegalArgumentException( "jsp.error.beans.propertyeditor.notregistered");
			}
		} 
		catch (IllegalArgumentException ex) 
		{
			throw new LassoException( "Bean property conversion error: " + ex.getMessage() 
				+ "(value=" + attrValue + ", class=" + attrClass.getName() + ", attribute=" + attrName + ")");
		}
	}
	
	/**
	 * Create a typed array.
	 * This is a special case where params are passed through
	 * the request and the property is indexed.
	 */	
	public static boolean createTypedArray (String prop, Object bean, Method method, String[] values, Class t, Class editor)
	{
		try 
		{
			if (editor != null) 
			{
				Object[] tmpval = new Integer[values.length];
				for (int i=0; i<values.length; i++) 
				{
					tmpval[i] = getValueFromBeanInfoPropertyEditor(t, prop, values[i], editor);
				}
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Integer.class)) 
			{
				Integer []tmpval = new Integer[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] =	new Integer (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Byte.class)) 
			{
				Byte[] tmpval = new Byte[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = new Byte (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Boolean.class)) 
			{
				Boolean[] tmpval = new Boolean[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = new Boolean (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Short.class)) 
			{
				Short[] tmpval = new Short[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = new Short (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Long.class)) 
			{
				Long[] tmpval = new Long[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = new Long (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Double.class)) 
			{
				Double[] tmpval = new Double[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = new Double (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Float.class)) 
			{
				Float[] tmpval = new Float[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = new Float (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(Character.class)) 
			{
				Character[] tmpval = new Character[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = new Character(values[i].charAt(0));
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(int.class)) 
			{
				int []tmpval = new int[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = Integer.parseInt (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(byte.class)) 
			{
				byte[] tmpval = new byte[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = Byte.parseByte (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(boolean.class))
			{
				boolean[] tmpval = new boolean[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = (Boolean.valueOf(values[i])).booleanValue();
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(short.class)) 
			{
				short[] tmpval = new short[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = Short.parseShort (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(long.class)) 
			{
				long[] tmpval = new long[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = Long.parseLong (values[i]);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(double.class)) 
			{
				double[] tmpval = new double[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = Double.valueOf(values[i]).doubleValue();
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(float.class)) 
			{
				float[] tmpval = new float[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = Float.valueOf(values[i]).floatValue();
				method.invoke (bean, new Object[] {tmpval});
			} 
			else if (t.equals(char.class)) 
			{
				char[] tmpval = new char[values.length];
				for (int i = 0 ; i < values.length; i++)
					tmpval[i] = values[i].charAt(0);
				method.invoke (bean, new Object[] {tmpval});
			} 
			else 
			{
				Object[] tmpval = new Integer[values.length];
				for (int i=0; i<values.length; i++) 
				{
					tmpval[i] =	getValueFromPropertyEditorManager( t, prop, values[i]);
				}
				method.invoke (bean, new Object[] {tmpval});
			}
		} 
		catch (Exception ex) 
		{
			return false;
		}
		return true;
	}
	
	public static Object convert(String propertyName, String s, Class t, Class propertyEditorClass) throws LassoException 
	{
		try 
		{
			if (s == null) 
			{
				if (t.equals(Boolean.class) || t.equals(Boolean.TYPE))
					s = "false";
				else
					return null;
			}
			
			if (propertyEditorClass != null) 
			{
				return getValueFromBeanInfoPropertyEditor(t, propertyName, s, propertyEditorClass);
			} 
			else if ( t.equals(Boolean.class) || t.equals(Boolean.TYPE) ) 
			{
				if (s.equalsIgnoreCase("on") || s.equalsIgnoreCase("true"))
					s = "true";
				else
					s = "false";
				return new Boolean(s);
			} 
			else if ( t.equals(Byte.class) || t.equals(Byte.TYPE) ) 
			{
				return new Byte(s);
			} 
			else if (t.equals(Character.class) || t.equals(Character.TYPE)) 
			{
				return s.length() > 0 ? new Character(s.charAt(0)) : null;
			} 
			else if ( t.equals(Short.class) || t.equals(Short.TYPE) ) 
			{
				return new Short(s);
			} 
			else if ( t.equals(Integer.class) || t.equals(Integer.TYPE) ) 
			{
				return new Integer(s);
			} 
			else if ( t.equals(Float.class) || t.equals(Float.TYPE) ) 
			{
				return new Float(s);
			} 
			else if ( t.equals(Long.class) || t.equals(Long.TYPE) ) 
			{
				return new Long(s);
			} 
			else if ( t.equals(Double.class) || t.equals(Double.TYPE) ) 
			{
				return new Double(s);
			} 
			else if ( t.equals(String.class) ) 
			{
				return s;
			} 
			else if ( t.equals(java.io.File.class) ) 
			{
				return new java.io.File(s);
			} 
			else if (t.getName().equals("java.lang.Object")) 
			{
				return new Object[] {s};
			} 
			else 
			{
				return getValueFromPropertyEditorManager( t, propertyName, s );
			}
		} 
		catch (Exception ex) 
		{
			throw new LassoException(ex);
		}
	}
	
	//*********************************************************************
	// Smart method selection
	//*********************************************************************
	
	public static Method selectMethod(java.util.Vector methods, Object args[]) 
	{
		if (methods.size() == 1) 
			return (Method)methods.firstElement();
	
		Object selected = null;
		int best = Integer.MAX_VALUE;
	
		for (java.util.Enumeration e = methods.elements(); e.hasMoreElements(); ) 
		{
			Object element = e.nextElement();
			int weight=0;
	
			Class parms[] = (element instanceof java.lang.reflect.Method) ?
				((java.lang.reflect.Method)element).getParameterTypes() : 
				((java.lang.reflect.Constructor)element).getParameterTypes();
	
			for (int i=0; i<parms.length; i++) 
			{
				if (parms[i].isInstance(args[i])) 
				{
					for (Class c=parms[i]; (c=c.getSuperclass()) != null; ) 
					{
						if (!c.isInstance(args[i])) 
							break;
						weight++;
					}
				}
				else if (parms[i].isAssignableFrom(java.lang.String.class)) 
				{
					if (!(args[i] instanceof byte[]) && !(args[i] instanceof String))
						weight+=9999;
				}
				else if (parms[i].isArray()) 
				{
					if (args[i] instanceof java.util.Hashtable)
						weight+=256;
					else
						weight+=9999;
				}
				else if (parms[i].isPrimitive()) 
				{
					Class c=parms[i];
					if (args[i] instanceof Number) 
					{
						if (c==Boolean.TYPE) weight+=5;
						if (c==Character.TYPE) weight+=4;
						if (c==Byte.TYPE) weight+=3;
						if (c==Short.TYPE) weight+=2;
						if (c==Integer.TYPE) weight++;
						if (c==Float.TYPE) weight++;
					}
					else if (args[i] instanceof Boolean) 
					{
						if (c!=Boolean.TYPE) weight+=9999;
					}
					else if (args[i] instanceof String) 
					{
						if (c== Character.TYPE || ((String)args[i]).length()>0)
							weight+=((String)args[i]).length();
						else
							weight+=64;
					}
					else 
					{
						weight+=9999;
					}
				}
				else 
				{
					weight+=9999;
				}
			}
	
			if (weight < best) 
			{
				if (weight == 0) 
					return (Method)element;
				best = weight;
				selected = element;
			}
		}
	
		return (Method)selected;
	}
	
}