package com.omnipilot.util;

import com.blueworld.lassopro.LassoJavaModule;
import java.lang.reflect.Modifier;

final class NativeLoader
{	
	final static String MODULES_DIR = "LassoModules/";	
	
	static String os = System.getProperty("os.name").toLowerCase( /*Locale.US*/ );
	static String ext = "";
	static String path = "";	
	
	static 
	{
		if ( os.startsWith("mac") )
		{
			path = System.getProperty("user.dir") + "/" + MODULES_DIR;
			ext = ".dylib";
		}	
		else if ( os.startsWith("linux") )
		{
			path = System.getProperty("user.dir") + "/" + MODULES_DIR;
			ext = ".so";
		}
	}
	
	public static boolean loadLibrary (java.lang.String name)
	{
		try
		{
			if ( os.startsWith("win") )
			{
				System.loadLibrary( name );	
			}
			else
			{
				// on MacOSX/Linux we load the library using the absolute filepath.
				try
				{
					System.load( path + name + ext );
				}
				catch(UnsatisfiedLinkError e)
				{
					System.load( path + "../../../LassoModules/" + name + ext );
				}
			}
			
			lasso_log("Done loading \"" + name + "\" native library.");
			return true;
		}
		catch(SecurityException e) 
		{	
			lasso_log("Native library loader error: operation not permitted.");
		}
		catch(UnsatisfiedLinkError e) 
		{
			lasso_log("Native library loader error: \"" + name + "\" library not found.");
		}
		
		return false;
	}
	
	private static void lasso_log(String message)
	{
		System.out.println( "[" + new java.text.SimpleDateFormat("MM/dd/yy HH:mm:ss").format(new java.util.Date()) + "] " + message );
	}
	
	private static void loadModule(String filename)
	{
		String name = filename.substring( 0, filename.lastIndexOf('.') ).replace('/', '.');
		if ( name.startsWith("com.blueworld.jdbc") || name.startsWith("com.omnipilot.jdbc") )
			return;
		else if ( name.startsWith("com.omnipilot.tags") )
			System.out.println("Loading module: " + name );
			
		Class clazz = null;
		try  {
			clazz = java.lang.Class.forName(name);
			
			if ( clazz != null && !clazz.isInterface() && !Modifier.isAbstract( clazz.getModifiers() ) 
			&& LassoJavaModule.class.isAssignableFrom( clazz ) )
			{
				lasso_log ( "Loading " + filename + " module...");
				LassoJavaModule module = (LassoJavaModule)clazz.newInstance();
				module.registerLassoModule();
			}
		}
		catch ( NoClassDefFoundError ncdfe ) {
			//lasso_log ( "Native Loader couldn't find class: " + name );
		}
		catch ( Exception ex ) {
			lasso_log ( "Native Loader received error: " + ex.getMessage() );
			ex.printStackTrace();
		}
		catch ( Throwable ie ) {
			lasso_log ( "Internal error while loading class " + name  + ": " + ie.getMessage() );
		}
	}
	
	public static void loadLassoModules() throws Exception
	{
		java.io.File dir = new java.io.File( MODULES_DIR );
		if (dir != null && dir.isDirectory() )
			loadLassoModules(dir);
		dir = new java.io.File(MODULES_DIR + "../../../" + MODULES_DIR);
		if (dir != null && dir.isDirectory() )
			loadLassoModules(dir);
	}
	
	public static void loadLassoModules(java.io.File dir)
	{	
		java.io.File files[] = dir.listFiles();
		if (files.length == 0)
			return;
		
		for (int i = 0; i < files.length; i++)
		{
			java.io.File file = files[i];
			if ( file.isDirectory() )
			{
				loadLassoModules(file);
				continue;
			}
			
			String name = file.getName();
			String ext = name.substring(name.lastIndexOf('.') + 1).toLowerCase();
			if ( ext.equalsIgnoreCase("class") )
			{
				loadModule(name);
			}
			else if ( ext.equalsIgnoreCase("jar") || ext.equalsIgnoreCase("zip") )
			{
				try {
					java.util.zip.ZipFile zip = new java.util.zip.ZipFile( file );
				
					for ( java.util.Enumeration e = zip.entries() ; e.hasMoreElements() ; )
					{
						name = ((java.util.zip.ZipEntry)e.nextElement()).getName();
						if ( name.endsWith(".class") )
							loadModule(name);
					}
				}
				catch ( Exception ze)
				{
					lasso_log ( "Native Loader Exception: " + ze.getMessage() );
				}
			}		
		}
	}
}