<?lassoscript

	define_type('smtp',
		-namespace = 'email_');

		local(
			'net' = net,
			'host' = 'localhost',
			'port' = 25,
			'timeout' = 15,
			'count' = 0);


		//->onCreate
		// nothing here yet
		define_tag('oncreate');
		/define_tag;


		// ->onConvert
		// We just print out the object host and port
		define_tag('onconvert');
			if(params->get(1) == 'string');
				return('SMTP (' self->'host' ':' self->'port' ')');
			/if;
		/define_tag;


		// ->onDestroy
		// We just print out the object host and port

		define_tag('ondestroy');
			protect;
				self->'net'->close;
			/protect;
		/define_tag;


		// ->Open
		// Open the connection and send basic commands to start the session
		define_tag('open',
			-required = 'host',
			-type = 'string',
			-optional = 'port',
			-type = 'integer',
			-optional = 'timeout',
			-type = 'integer',
			-optional = 'username',
			-type = 'string',
			-optional = 'password',
			-type = 'string');

			local(
				'net' = @self->'net',
				'auth' = false,
				'authtype' = '',
				'ehlo' = false,
				'commands' = string);

			if(global_defined('__smtp_ip__') && $__smtp_ip__->size);
				local('__local_smtp_ip__' = $__smtp_ip__);
			else(global('__lassoservice_ip__')->size);
				local('__local_smtp_ip__' = $__lassoservice_ip__);
			else;
				local('__local_smtp_ip__' = '127.0.0.1');
			/if;

			if(local_defined('username') && local_defined('password') && #username->size && #password->size);
				#auth = true;
			else(local_defined('username') || local_defined('password'));

				if(local_defined('username') && local_defined('password') && (!#username->size) && (!#password->size));
					error_code = -1;
					error_msg = '[Email_SMTP] Username and Password were empty or null';
					return(false);
				else((local_defined('username') && (!#username->size)) || (!local_defined('username')));
					error_code = -1;
					error_msg = '[Email_SMTP] Username was empty or null';
					return(false);
				else((local_defined('password') && (!#password->size)) || (!local_defined('password')));
					error_code = -1;
					error_msg = '[Email_SMTP] Password was empty or null';
					return(false);
				/if;

			/if;

			self->'host' = #host;

			if(local_defined('port') && (#port->type == 'integer'));
				self->'port' = #port;
			/if;

			if(local_defined('timeout'));
				self->'timeout' = #timeout;
			else;
				self->'timeout' = 60;
			/if;

			local_remove('host');
			local_remove('port');
			local_remove('timeout');

			local(
				'host' = @self->'host',
				'port' = @self->'port',
				'timeout' = @self->'timeout');

			// test EHLO [ ... ]
			if(#net->connect(#host, #port) == net_connectok);
				if(!self->command(-expect = 220));
					return(false);
				/if;

				#commands = self->command(-send = 'EHLO [' + #__local_smtp_ip__ + ']\r\n', -expect = 250, -read);
				if(#commands !== false);
					if(#commands->find('\r\n'));
						local('eol' = '\r\n');
					else(#commands->find('\n'));
						local('eol' = '\n');
					else;
						local('eol' = '\r');
					/if;

					#ehlo = true;

					if(#auth);
						iterate(#commands->split(#eol), local('i'));
							if(!#authtype->size);
								if((#i >> 'AUTH ') && (#i >> 'CRAM-MD5'));
									#authtype = 'cram-md5';
								else((#i >> 'AUTH ') && (#i >> 'LOGIN'));
									#authtype = 'login';
								/if;
							/if;
						/iterate;

						if(!#authtype->size);
							#authtype = 'login';
						/if;
					/if;
				else;
					#net->close;
				/if;

			/if;

			if(!#ehlo);
				if(#net->connect(#host, #port) == net_connectok);
					if(!self->command(-expect = 220));
						return(false);
					/if;

					if(!self->command(-send = 'HELO [' + #__local_smtp_ip__ + ']\r\n', -expect = 250));
						#net->close;
						return(false);
					/if;

				else;
					#net->close;
					error_code = -8001;
					error_msg = '[Email_SMTP] Could not connect to server';
					return(false);
				/if;
			/if;

			// add auth stuff here
			if(#auth);
				if(#authtype == 'cram-md5');
					local(
						'authresp' = self->command(-send = 'AUTH CRAM-MD5\r\n', -expect = 334, -read),
						'cram' = null);
					if(#authresp !== false);
						#authresp->trim;
						#authresp = string(decode_base64(#authresp->split(' ')->get(2)));
						#cram = encode_base64(#username ' ' encrypt_crammd5(#password, #authresp));

						if(!self->command(-send = #cram + '\r\n', -expect = 235));
							#net->close;
							return(false);
						/if;
					else;
						#net->close;
						return(false);
					/if;
				else(#authtype == 'login');
					local(
						'auth_user' = encode_base64(#username),
						'auth_pass' = encode_base64(#password));

					if(!self->command(-send = 'AUTH LOGIN\r\n', -expect = 334));
						#net->close;
						return(false);
					/if;

					if(!self->command(-send = #auth_user + '\r\n', -expect = 334));
						#net->close;
						return(false);
					/if;

					if(!self->command(-send = #auth_pass + '\r\n', -expect = 235));
						#net->close;
						return(false);
					/if;
				/if;
			/if;

			return('Auth: ' #auth);

			return(true);
		/define_tag;


		// ->Command
		// This will send a command to the SMTP server, and return the result if bad
		define_tag('command',
			-optional = 'send',
			-optional = 'expect',
			-optional = 'read');

			local(
				'net' = @self->'net',
				'host' = @self->'host',
				'port' = @self->'port',
				'timeout' = @self->'timeout',
				'result' = null,
				'bytes' = bytes);

			if(!local_defined('expect'));
				local('expect' = null);
			/if;

			if(local_defined('send') && #send->size);

				#result = #net->wait(#timeout, net_waitwrite);
				if(#result->isa('integer') && (#result == net_waittimeout));
					error_code = -8001;
					error_msg = '[Email_SMTP] SMTP server time out waiting for write';
					return(false);
				/if;

				// convert text to iso-8859-1 character set
				#bytes->importas(#send, 'iso8859-1');

				null(#net->write(#bytes));
			/if;

			#result = #net->wait(#timeout, net_waitread);
			if(#result->isa('integer') && (#result == net_waittimeout));
				error_code = -8001;
				error_msg = '[Email_SMTP] SMTP server time out waiting for read';
				return(false);
			/if;

			#result = #net->read(32768);

			if(#result->size);
				local('answerlist' = integer(string_findregexp(#result, -find='[0-9]+')->get(1)));
				error_seterrormessage('[Email_SMTP] Server Error: ' #result);
				if(#answerlist != #expect);
					error_code = #answerlist;
					error_seterrormessage('[Email_SMTP] Server Error: ' #result);
					return(false);
				else;
					error_code = 0;
				/if;
			else;
				error_msg = '[Email_SMTP] SMTP server sent empty response';
				error_code = -8001;
				return(false);
			/if;

			if(local_defined('read'));
				return(#result);
			else;
				return(true);
			/if;

		/define_tag;


		// ->Send
		// We send the message here
		define_tag('send',
			-required = 'from',
			-type = 'string',
			-required = 'recipients',
			-type = 'array',
			-required = 'message');

			local(
				'net' = @self->'net',
				'timeout' = @self->'timeout');

			if(!self->command(-send = 'RSET\r\n', -expect = 250));
				return(false);
			/if;

			if(!self->command(-send = 'MAIL FROM:<' #from '>\r\n', -expect = 250));
				return(false);
			/if;

			iterate(#recipients, local('i'));
				if(!self->command(-send = 'RCPT TO:<' #i '>\r\n', -expect = 250));
					return(false);
				/if;
			/iterate;

			if(!self->command(-send = 'DATA\r\n', -expect = 354));
				return(false);
			/if;

			if(#net->wait(#timeout, net_waitwrite) == net_waitwrite);
				#net->write(#message);
			/if;

			if(!self->command(-send = '\r\n.\r\n', -expect = 250));
				return(false);
			/if;

			return(true);
		/define_tag;


		// ->Close
		// Send QUIT and close the connection
		define_tag('close');
			local('net' = @self->'net');
			protect;
				self->command(-send = 'QUIT\r\n', -expect = 221);
			/protect;
			protect;
				#net->close;
				return(true);
			/protect;

			return(false);
		/define_tag;

	/define_type;

?>