<?LassoScript
	library:lassoapp_link:'session_drivers.lasso';
	/* 
		sessiontracker script modified by Johan Solve 2003-10-10
		based on 7.0b5
		I'm going to add -Path as optional params for 
		session_start, and then I'm going to set the default cookie to 
		-expires+1440 instead of just -expires to make it less 
		sensitive to clock skew. 
		If -CookieExpires is specified but empty, use temp session cookie. 
		I'm also going to make session_end delete the session cookie if 
		there is one. 
		
		session_start
			Added optional -path parameter, defaults to '/' if not specified
			If -CookieExpires is set to '' explicitly, use a "real" ression cookie without expiration
			If -CookieExpires is not specified, use session expiration + 24 hours for the 
			  default cookie expiration. This is to make it less vurnerable to clock skew. 

		session_end
			added code to delete session cookie if there is one. Cookie is deleted by setting a blank cookie with 
			  large negative expiration, to make sure it gets deleted even with clock skew

	*/

	// the current session driver
	session_setdriver;
	
	// This startup script is enclosed in a compound expression so the variables will not
	// show up in the list of global variables.
	{
		Auth_Admin;

		Log_Warning: 'Sessions: Initializing';
		
		// create the site's sessions db
		// special DS name
		Local: 'table_exists' = false;
		inline: -log=(global: '__startup_loglevel__'), -Database='site', -SQL='SELECT * from sqlite_master WHERE type="table" AND name="sessions"', -MaxRecords=1;
			if: (found_count) > 0;
				#table_exists = true;
			/if;
		/inline;
		if: (#table_exists == false);
			var: '__need_reload__' = true;
			$_session_driver->createTable('site');
		/if;

		inline: -log=$__startup_loglevel__, -database='lasso_internal', -table='global_prefs';
			
		inline: -log=$__startup_loglevel__, -search, -op='eq', 'store_key'='_session_globalattributes', -returnfield='data';

			if: (found_count > 0) && ((field: 'data') != '');
				protect;
					local: 'data' = (field: 'data');
					local: 'temp' = null;
					#temp->(unserialize: #data);
					if: (#temp != null) && (#temp->type == 'array');
						global: '_session_globalattributes' = #temp;
					/if;
				/protect;
			/if;
		/inline;
		
		if: ($_session_globalattributes == null) || ($_session_globalattributes == array);
			global: '_session_globalattributes' = (array: 'form'='action','a'='href');
		/if;
			
		inline: -log=$__startup_loglevel__, -search, -op='eq', 'store_key'='_session_globalignoreprefixes', -returnfield='data';

			if: (found_count > 0) && ((field: 'data') != '');
				protect;
					local: 'data' = (field: 'data');
					local: 'temp' = null;
					#temp->(unserialize: #data);
					if: (#temp != null) && (#temp->type == 'array');
						global: '_session_globalignoreprefixes' = #temp;
					/if;
				/protect;
			/if;
		/inline;
		
		if: ($_session_globalignoreprefixes == null) || ($_session_globalignoreprefixes == array);
			global: '_session_globalignoreprefixes' = (array: 'javascript:','mailto:','https://','ftp://','telnet://','file://','#','http://');
		/if;

		inline: -log=$__startup_loglevel__, -search, -op='eq', 'store_key'='lasso_session_database', -returnfield='data';
			if:found_count > 0;
				(global:'_lasso_session_database' = string:field:'data');
			else;
				(global:'_lasso_session_database' = 'site');
			/if;
		/inline;
		
		inline: -log=$__startup_loglevel__, -search, -op='eq', 'store_key'='lasso_session_table', -returnfield='data';
			if:found_count > 0;
				(global:'_lasso_session_table' = string:field:'data');
			else;
				(global:'_lasso_session_table' = 'sessions');
			/if;
		/inline;
		
		/inline;

		if: ($_lasso_session_database != 'site') || ($_lasso_session_table != 'sessions');
			session_setdriver;
		/if;
		
		// Check the schema of the session database
		local: 'session_error' = ($_session_driver->validateSessionsTable);

		// Reset the session storage if there is an error
		if: (#session_error == false);
			Log_Critical: 'Sessions: Error: Resetting to Local Session Storage';
			(global:'_lasso_session_database' = 'site');
			(global:'_lasso_session_table' = 'sessions');
			session_setdriver;
		else: ($_lasso_session_database == 'site') && ($_lasso_session_table == 'sessions');
			Log_Warning: 'Sessions: Using Local Session Storage';
		else;
			Log_Warning: 'Sessions: Using External Session Storage "' + $_lasso_session_database + '"."' + $_lasso_session_table + '"';
		/if;

	}->(run); 

	// Fix encoding on sessions table
	inline: -log=$__startup_loglevel__, -search,
			-database='lasso_internal',
			-table='security_datasource_databases',
			-keyfield='id',
			-op='eq', 'name'=(database_realname: (global:'_lasso_session_database')),
			-maxrecords='all',
			-returnfield='id';
		records;
			inline: -log=$__startup_loglevel__, -search,
					-database='lasso_internal',
					-table='security_database_tables',
					-keyfield='id',
					-op='eq', 'id_database'=(field: 'id'),
					-op='eq', 'name'=(table_realname: (global:'_lasso_session_database'), (global:'_lasso_session_table')),
					-op='neq', 'encoding'='utf-8',
					-maxrecords='all',
					-returnfield='id';
				records;
					inline: -log=$__startup_loglevel__, -update, 
							-database='lasso_internal',
							-table='security_database_tables',
							-keyfield='id',
							-keyvalue=(field: 'id'),
							'encoding'='utf-8',
							-maxrecords=0;
					/inline;
				/records;
			/inline;
		/records;
	/inline;
	
namespace_using:'session';

	define_tag:'start', -priority='replace',
					-required='name', -type='string', 
					-optional='expires',
					-optional='usecookie',
					-optional='uselink',
					-optional='useauto',
					-optional='usenone',
					-optional='id', -type='string',
					-optional='domain', -type='string',
					-optional='path', -type='string', 		
					-optional='cookieexpires',
					-optional='rotate';
		
		var: '_SessionTracker_Result' = 'load'; // Values include new/expire/load/rotate
	
		local:'sessionVarsKey' = ('_SessionTracker_' #name '_Vars');
		if:var_defined:#sessionVarsKey; // already started
			return;
		/if;
		
		if:!local_defined:'expires';
			local:'_expires'= 15;
		else;
			local:'_expires'=(integer: #expires);
		/if;					
	
		// added by JS 2003-10-10
		if:!local_defined:'path';
			local:'_path'= '/';
		else;
			local:'_path'=(string: #path);
		/if;					
					
		(var:#sessionVarsKey) = :;
		local:'data'=null;
				
		if:local_defined:'id'; // force the specified id
			var:('_SessionTracker_' #name) = #id;
		else;
			if:!var_defined:('_SessionTracker_' #name);
				local:'id' = cookie:('_SessionTracker_' #name);				
				var:('_SessionTracker_' #name) = #id;
			else;
				local:'id' = var:('_SessionTracker_' #name);
			/if;
		/if;
		
		if:#id->size == 0;
			// this is a new session that is starting up
			local:'id' = lasso_uniqueid;					
			var:('_SessionTracker_' #name) = #id;
			$_session_driver->init('_SessionTracker_' #name '_' #id, #_expires);
			var: '_SessionTracker_Result' = 'new'; // Values include new/expire/load/rotate
		/if;

		local: 'data' = $_session_driver->fetchData('_SessionTracker_' #name '_' #id);
			
		if: (#data->type == 'null') || (local_defined: 'rotate');
			$_session_driver->kill('_SessionTracker_' #name '_' #id);
			local:'id' = lasso_uniqueid; // Create new ID if passed ID is invalid				
			var:('_SessionTracker_' #name) = #id;
			$_session_driver->init('_SessionTracker_' #name '_' #id, integer:#_expires);
			if: (#data->type != 'null') && (local_defined: 'rotate');
				var: '_SessionTracker_Result' = 'rotate'; // Values include new/expire/load/rotate
			else: ($_SessionTracker_Result >> 'load');
				var: '_SessionTracker_Result' = 'expire'; // Values include new/expire/load/rotate
			/if;
		/if;
				
		if:#data->size;
			// there is some data
			protect;
				local:'xml' = xml:#data;
				// hack: this will fix the & in the data if there is an
				// error reported by the xml operation.
				handle_error;
					#data->(replace: '&amp;', '--LKJDFOEIOWMNCLOIU--');
					#data->(replace: '&', '&amp;');
					#data->(replace: '--LKJDFOEIOWMNCLOIU--', '&amp;');
					local: 'xml' = xml: #data;
					error_code = 0;
					error_msg = '';
				/handle_error;
			/protect;
			
			local:'names'=#xml->extract:'/session/var/@key';
			local:'values'=#xml->extract:'/session/var/*'; /* */
			local:'reg'=null;		
			iterate:#names, local:'i';
				#reg = (string_findregexp:-find='key="(.*)"', #i);
				(var:#sessionVarsKey)->insert:(#reg->get:2);
			/iterate;
			iterate:var:#sessionVarsKey, local:'i';
				(var:#i) = null;
				(var:#i)->unserialize:(#values->get:loop_count);
			/iterate;
		/if;
	
		if:#_expires == -1;
			#_expires = 365 * 1440;
		/if;
		local: 'usesessioncookie'=false;
		if:local_defined:'cookieexpires';
			if: (string: #cookieexpires) == ''; 
				// cookieexpires explicitly set to '', so use a session cookie without expiration
				// JS 2003-10-10
				#usesessioncookie = true;
			else;
				#_expires = (integer: #cookieexpires);
			/if;
		else;
			// use session expiration for cookie expiration as default, 
			// but add 24 hours to give headroom for clock skew
			// JS 2003-10-10
			#_expires = #_expires + 24 * 60;
		/if;
	
		// perpetuation
		if: (local_defined:'uselink') || ( (local_defined: 'useauto') && (client_cookies !>> ('_SessionTracker_' #name)));
			// install the link rewriter
			session_addoutputfilter:#name=#id;
		/if;
		// default is usecookie
		if: (local_defined:'usecookie') || (local_defined: 'useauto') || ( (!local_defined:'uselink') && (!local_defined:'usenone') );
				// merged the logic for cookie_set   JS 2003-10-10
			if: #usesessioncookie;
				// set cookie without expiration
				cookie_set:('_SessionTracker_' #name)=#id, 
					-domain=(local:'domain'), 
					-path=#_path;
			else;
				// set cookie with expiration
				cookie_set:('_SessionTracker_' #name)=#id, 
					-expires=#_expires, 
					-domain=(local:'domain'), 
					-path=#_path;
			/if;
		/if;
			
		// install the var saver
		$_at_end->insert: { 
								local:'name'=@params;
								if:(var_defined:'_SessionTracker_' #name);
									local:'id'=(var:'_SessionTracker_' #name);
									
									local:'xml'='<?xml version="1.0" ?>\r\n';
									#xml += '<session id="' #id '">\r\n';
									
									iterate:var:('_SessionTracker_' #name '_Vars'), local:'i';
										#xml += '<var key="' #i '">';
										#xml += (var:#i)->serialize;
										#xml += '</var>';
									/iterate;
									
									#xml += '</session>';
									$_session_driver->saveData('_SessionTracker_' #name '_' #id, #xml);
								/if;
							}=#name;
		if:(math_random:-min=0, -max=20) == 0;
			$_session_driver->expire;
		/if;
		
	/define_tag;
	
	define_tag:'addvar', -priority='replace',
					-required='name', -type='string';
		local:'tmpary' = @(var:'_SessionTracker_' #name '_Vars');
		if: #tmpary->(isa: 'array');
			iterate: params, (local: 'tmp');
				if: #tmp->isa('pair');
                    loop_continue;
			    /if;
				fail_if: !#tmp->isa('string'), error_code_invalidparameter, '[Session_AddVar]: Variable names must be of type string: ' (string: #tmp);
				if: #tmpary !>> #tmp;
					#tmpary->(insert: #tmp);
				/if;
			/iterate;
		/if;
	/define_tag;

	namespace_import: 'session_addvar', 'session_addvariable';
	
	define_tag:'removevar', -priority='replace',
					-required='name', -type='string';
		local:'tmpary' = @(var:'_SessionTracker_' #name '_Vars');
		if: #tmpary->(isa: 'array');
			iterate: params, (local: 'tmp');
				if: !(#tmp->(isa: 'pair'));
					#tmpary->(removeall: #tmp);
				/if;
			/iterate;
		/if;
	/define_tag;

	namespace_import: 'session_removevar', 'session_removevariable';
		
	define_tag:'end', -priority='replace', 
					-required='name', -type='string';
		local:'id'= @(var:'_SessionTracker_' #name);
		$_session_driver->kill('_SessionTracker_' #name '_' #id);
		var_remove:'_SessionTracker_' #name;
		var_remove:'_SessionTracker_' #name '_Vars';
		// delete session cookie if there is one, use large negative expiration to make 
		// sure it is deleted even with clock skew   JS 2003-10-10
		if: (cookie: ('_SessionTracker_' #name)) != '';
			cookie_set:('_SessionTracker_' #name)='', 
				-expires=(-32767),
				-path='/';
				// Lasso can't seem to handle larger negative expiration than this
		/if;
	/define_tag;
	
	define_tag:'id', -priority='replace',
					-required='name', -type='string';
		if: !var_defined:('_SessionTracker_' #name);
			return;
		/if;
		return:var:('_SessionTracker_' #name);
	/define_tag;
		
	define_tag:'abort', -priority='replace',
					-required='name', -type='string';
		var_remove:('_SessionTracker_' #name);
	/define_tag;	
		
	define_tag:'result', -priority='replace';
		return: (string: (var: '_SessionTracker_Result'));
	/define_tag;	

/namespace_using;

	// Delete Expired Sessions
	$_session_driver->expire;
	
?>
