<?LassoScript

define_tag: 'Uploads', 
		-namespace='_global_File_',
		-optional='raw',
		-priority='replace';
		
	if: !(var_defined:'_uploads_');
		return: array;
	/if;
	
	if: (local_defined: 'raw');
		return: @$_uploads_;
	/if;
	
	local: 'output' = $_uploads_;
	
	iterate: #output, (local: 'temp');
	
		local: 'path' = #temp->(find: 'upload.name');
		local: 'size' = (integer: #temp->(find: 'upload.size'));
		local: 'type' = (string: #temp->(find: 'upload.type'));
		local: 'param' = (string: #temp->(find: 'upload.fieldname'));
		local: 'file' = null;
		local: 'tempname' = #path->(split: '/')->last->(split: '\\')->last;
		protect;
			local: 'file' = (file: #path);
			local: 'size' = #file->size;
		/protect;
		local: 'orig' = #temp->(find: 'upload.realname');
		#orig->trim;
		local: 'origname' = #orig->(split: '/')->last->(split: '\\')->last;
		local: 'origpath' = #orig->(removetrailing: #origname);
		if: (#origname >> '.');
			local: 'origextension' = (string_replaceregexp: #origname, -find='.*\\.', -replace='');
		else;
			local: 'origextension' = '';
		/if;		
		
		#temp->(insert: 'path' = #path);
		#temp->(insert: 'file' = #file);
		#temp->(Insert: 'size' = #size);
		#temp->(insert: 'type' = #type);
		#temp->(insert: 'param' = #param);
		#temp->(insert: 'origname' = #origname);
		#temp->(insert: 'origpath' = #origpath);
		#temp->(insert: 'origextension' = #origextension);
	
	/iterate;
	
	return: @#output;
	
/define_tag;

define_tag: 'ProcessUploads',
		-namespace='_global_File_',
		-required='destination',
		-optional='maxsize',
		-optional='extensions',
		-optional='fileoverwrite',
		-optional='usetempnames';
	
	local: '_uploads' = file_uploads;
	
	if: (#_uploads->size == 0);
		return: '';
	/if;
	
	fail_if: !(#_uploads->first->(find: 'file')->(isa: 'file')), -1, 'No permission to access uploaded files.';
			
	if: (local_defined: 'extensions') && (#extensions->(isa: 'array'));
		local: '_extensions' = #extensions;
	else: (local_defined: 'extensions') && (#extensions->(isa: 'string'));
		local: '_extensions' = #extensions->(split: ',');
		iterate: #_extensions, (local: 'temp');
			#temp->trim;
			if: (#temp->(beginswith: '.'));
				#temp -= '.';
			/if;
		/iterate;
	else;
		local: '_extensions' = (array: 'txt','text','htm','html','xml','xsd','pdf','jpg','gif','png','bmp','psd','tif','rgb','cmyk');
	/if;

	local: '_maxsize' = (integer: (local: 'maxsize'));
	if: #_maxsize == 0;
		local: '_maxsize' = 1024 * 1024 * 1024;
	/if;

	local: '_destination' = (string: #destination);
	#_destination->trim;
	if: !(#_destination->(endswith: '/'));
		#_destination += '/';
	/if;
	if: !(file_exists: #_destination);
		file_create: #_destination;
	/if;
	fail_if: !(file_exists: #_destination), -1, 'The destination folder "' + #_destination + '" doesn\'t exist and couldn\'t be created.';

	fail_if: (file_currenterror != error_noerror), (file_currenterror: -errorcode), (file_currenterror);

	iterate: #_uploads, (local: 'temp');
	
		if: (#temp->(find: 'size') > #_maxsize);
			loop_continue;
		/if;
		if: (#_extensions !>> #temp->(find: 'origextension'));
			loop_continue;
		/if;
		if: (local_defined: 'usetempnames');
			file_copy: #temp->(find: 'path'), #_destination + #temp->(find: 'tempname');
			fail_if: (file_currenterror != error_noerror), (file_currenterror: -errorcode), (file_currenterror);
		else;
			if: (local_defined: 'fileoverwrite');
				file_copy: #temp->(find: 'path'), #_destination + #temp->(find: 'origname'), -fileoverwrite;
				fail_if: (file_currenterror != error_noerror), (file_currenterror: -errorcode), (file_currenterror);
			else;
				file_copy: #temp->(find: 'path'), #_destination + #temp->(find: 'origname');
				fail_if: (file_currenterror != error_noerror), (file_currenterror: -errorcode), (file_currenterror);
			/if;
		/if;
		
	/iterate;
	
/define_tag;

define_tag:'ProbeEOL', -required='buff', -returntype='bytes', -namespace = '_global_File_';
	iterate:#buff, local:'char';
		if:#char == 0x000A;
			return:bytes:'\n';
		else if:#char == 0x000D;
			if:#buff->get(loop_count + 1) == 0x000A; // \r\n
				return:bytes:'\r\n';
			else; // \r
				return:bytes:'\r';
			/if;
		/if;
	/iterate;
/define_tag;

define_tag:'ReadLine',
		-namespace='_global_File_',
		-required='file',
		-required='FileLineNumber',
		-optional='fileendofline',
		-priority='replace';

	local:'buff' = file_read:#file;
	if:file_currenterror(-errorcode) != 0 || #buff->size == 0;
		return:0;
	/if;

	local:'myeol'=null;
	if:!local_defined('fileendofline');
		#myeol = File_probeEOL:#buff;
	else;
		#myeol = bytes:#fileendofline;
	/if;

	local:'count' = 1; // at least one line
	local:'lastLastPos'=1;
	local:'lastPos' = #buff->find(#myeol);
	if:#lastPos == 0; // no line end found, however, the file had data, so return it all
		return:string(#buff);
	/if;
	while:#lastPos != 0 && #count < #FileLineNumber;
		#lastLastPos = #lastPos;
		#lastPos = #buff->find(#myeol, #lastPos + #myeol->size);
		if: #lastPos != 0;
			#count += 1;
		else;
			#lastPos = #buff->size;
			#count += 1;
			if:#count < #FileLineNumber;
				return:'';
			/if;
		/if;
		if:#lastLastPos != 1;
			#lastLastPos += #myeol->size;
		/if;
	/while;

	// if the start position is at the end of the buffer, just return empty
	if: #lastPos == #buff->size;
		return: '';
	/if;

	return:string(#buff->getrange(#lastLastPos, #lastPos - #lastLastPos));

/define_tag;

define_tag:'GetLineCount',
		-namespace='_global_File_',
		-required='file',
		-optional='fileendofline',
		-priority='replace';

	local:'buff' = file_read:#file;
	if:file_currenterror(-errorcode) != 0 || #buff->size == 0;
		return:0;
	/if;

	local:'myeol'=null;
	if:!local_defined('fileendofline');
		#myeol = File_probeEOL:#buff;
	else;
		#myeol = bytes:#fileendofline;
	/if;
	local:'count' = 1; // at least one line
	local:'lastPos' = #buff->find(#myeol);

	while:#lastPos != 0;
		#lastPos = #buff->find(#myeol, #lastPos + #myeol->size);
		#count += 1;
	/while;
	return:#count;
/define_tag;

//
// [File_Chmod]
//
// [File_Chmod: 'filename', Decimal Value]
// [File_Chmod: 'filename', Hexadecimal Value]
// [File_Chmod: 'filename', 'Quoted Octal Value']
// [File_Chmod: 'filename', -a='rwxst']
// [File_Chmod: 'filename', -a='rwxst', -u='rwx', -g='rwx', -o='rwx']
//

define_tag: 'ChMod',
		-namespace='_global_File_',
		-required='file',
		-optional='a',
		-optional='u',
		-optional='g',
		-optional='o';

	if: !(local_defined: 'a');
		local: 'a' = '';
	/if;

	local: 'perms' = 0;

	if: #a->(isa: 'string') && ((integer: #a) > 0);

		local: 'pow' = (integer: (math_pow: 8, #a->size - 1));
		iterate: #a, (local: 'char');
			#perms += (#pow * (integer: #char));
			#pow /= 8;
		/iterate;

	else: #a->(isa: 'integer');

		local: 'perms' = #a;

	else;
		if: (#a >> 'r');
			#perms += 292; // 0444
		/if;
		if: (#a >> 'w');
			#perms += 146; // 0222
		/if;
		if: (#a >> 'x');
			#perms += 73; // 0111
		/if;
		if: (#a >> 's');
			#perms += 3072; // 6000
		/if;
		if: (#a >> 't');
			#perms += 512; // 1000
		/if;
		if: (local_defined: 'u');
			if: (#u >> 'r') && (#a !>> 'r');
				#perms += 256; // 0400
			/if;
			if: (#u >> 'w') && (#a !>> 'w');
				#perms += 128; // 0200
			/if;
			if: (#u >> 'x') && (#a !>> 'x');
				#perms += 64; // 0100
			/if;
		/if;
		if: (local_defined: 'g');
			if: (#g >> 'r') && (#a !>> 'r');
				#perms += 32; // 0040
			/if;
			if: (#g >> 'w') && (#a !>> 'w');
				#perms += 16; // 0020
			/if;
			if: (#g >> 'x') && (#a !>> 'x');
				#perms += 8; // 0010
			/if;
		/if;
		if: (local_defined: 'o');
			if: (#o >> 'r') && (#a !>> 'r');
				#perms += 4; // 0004
			/if;
			if: (#o >> 'w') && (#a !>> 'w');
				#perms += 2; // 0002;
			/if;
			if: (#o >> 'x') && (#a !>> 'x');
				#perms += 1; // 0001
			/if;
		/if;
	/if;

	_file_chmod: #file, #perms;

/define_tag;



// [..._Serve] tags.
//
// These tags serve a 'file' in place of the output of the current format file.  They can be used instead
// of a [Redirect_URL] to serve a different file, can be used to serve a PDF or XML file created in memory,
// or can be used to serve an image file from a database field.
//
// Each tag sets the contents of the page in __http_header__ to an appropriate value and then resets the
// headers of the page with the file name and MIME type.  Finally, [Abort] is called to prevent further
// processing of the page.
//
// 'content' - The first parameter is the content of the file to be served.
// -File - Optional parameter which specifies the name of the file to be served.
// -Type - Optional parameter which specifies the MIME type of the content to be served.

Define_Tag: 'Serve',
		-namespace='_global_File_',
		-Priority='Replace',
		-Required='content',
		-Optional='file',
		-Optional='type';

	var: '__http_header__' = string;

	$__http_header__ += 'HTTP/1.0 200 OK\r\n';
	$__http_header__ += 'Server: Lasso/' + (lasso_version: -lassoversion) + '\r\n';
	$__http_header__ += 'MIME-Version: 1.0\r\n';

	if: (local_defined: 'type');
		$__http_header__ += 'Content-Type: ' + (string: #type) + '\r\n';
	else;
		$__http_header__ += 'Content-Type: text/html\r\n';
	/if;

	if: (local_defined: 'file') && (#file != '');
		local: 'safefile' = #file;
		#safefile->(replace: '"', '\\"');
		$__http_header__ += 'Content-Disposition: attachment; filename="' + #safefile + '"\r\n';
	/if;
	$__html_reply__ = #content;
	$__http_header__ += 'Content-Length: ' + $__html_reply__->size + '\r\n';

	Abort;

/Define_Tag;



/**************************************************
*
* Author: Adam Randall
* Company: Blue World Communications
* Date: 6/15/2004
* Version: 1.1
*
* This tag will allow you to stream a very large
* file, while utilizing a very small amount of
* resources. Load on the server should be minimal
*
**************************************************/

define_tag: 'Stream',
	-namespace = '_global_File_',
	-required = 'file',
	-optional = 'name',
	-optional = 'type',
	-optional = 'buffer',
	-type = 'integer',
	-priority = 'replace';

	// this tag should never time out
	lasso_executiontimelimit: 0;

	local: 'file_size' = (file_getsize: #file);

	if: ! (local_defined: 'buffer');
		local: 'buffer' = 30000;
	/if;

	if: !(local_defined: 'type');
		local: 'type' = 'application/octet-stream';
	/if;

	if: !(local_defined: 'name');
		local: 'safename' = #file->(split: '/')->last;
	else;
		local: 'safename' = #name;
	/if;
	#safename->(replace: '"', '\\"');

	if: (file_currenterror) == 'No error';

		$__http_header__ = 'HTTP/1.0 200 OK\r\n';
		$__http_header__ += 'Server: Lasso/' + (lasso_version: -lassoversion)->(get: 1) + '\r\n';
		$__http_header__ += 'MIME-Version: 1.0\r\n';
		$__http_header__ += 'Content-Type: ' + #type + '\r\n';
		$__http_header__ += 'Content-Disposition: attachment; filename="' + #safename + '"\r\n';
		$__http_header__ += 'Content-Length: ' + #file_size + '\r\n';

	loop:
		-loopfrom = 0,
		-loopto = ((math_ceil: #file_size / (decimal: #buffer)) - 1),
		-loopincrement = 1;

			local: 'start' = (loop_count) * #buffer;
			local: 'end' = #start + #buffer;

			if: #end > #file_size;
				#end = #file_size;
			/if;

			$__html_reply__ = (file_read:
				#file,
				-filestartpos = #start,
				-fileendpos = #end);

			server_push;

		/loop;

		server_push; // just in case
		abort;

	else;
		fail: -1, 'No permission to file';
	/if;

/define_tag;

?>